@@ -2,25 +2,10 @@
 #define _CRT_RAND_S
 
 #include <bson.h>
-#include <bcon.h>
-#include <fcntl.h>
-#include <stdio.h>
-#include <bson-string.h>
-#include <bson-private.h>
 #include <math.h>
 
-#include "bson-config.h"
-#include "bson-tests.h"
 #include "TestSuite.h"
 
-#ifndef BINARY_DIR
-#define BINARY_DIR "tests/binary"
-#endif
-
-#ifndef JSON_DIR
-#define JSON_DIR "tests/json"
-#endif
-
 static ssize_t
 test_bson_json_read_cb_helper (void *string, uint8_t *buf, size_t len)
 {
@@ -39,61 +24,57 @@ test_bson_json_allow_multiple (void)
    int32_t one;
    bson_error_t error;
    bson_json_reader_t *reader;
-   bson_json_reader_t *reader_multiple;
-   bson_t bson = BSON_INITIALIZER;
-   bool allow_multiple = true;
-   char *multiple_json = "{\"a\": 1}{\"b\": 1}";
+   bson_t bson;
+   const char *collection;
+   const bson_oid_t *oid;
+   bson_oid_t oid_expected;
+   int i;
+   /* nested JSON tests code that reuses parser stack frames, CDRIVER-2524 */
+   char *multiple_json =
+      "{\"a\": 1}"
+      "{\"b\": {\"$dbPointer\": "
+      "  {\"$ref\": \"c\", \"$id\": {\"$oid\": \"12341234123412abcdababcd\"}}}}"
+      "{\"c\": {\"x\": 1}}";
 
-   reader = bson_json_reader_new (
-      multiple_json, &test_bson_json_read_cb_helper, NULL, !allow_multiple, 0);
-   reader_multiple = bson_json_reader_new (
-      multiple_json, &test_bson_json_read_cb_helper, NULL, allow_multiple, 0);
-   BSON_ASSERT (reader);
-   BSON_ASSERT (reader_multiple);
+   bson_oid_init_from_string (&oid_expected, "12341234123412abcdababcd");
 
-   /* reader with allow_multiple false */
-   /* read first json */
-   ASSERT_CMPINT (1, ==, bson_json_reader_read (reader, &bson, &error));
-   BCON_EXTRACT (&bson, "a", BCONE_INT32 (one));
-   ASSERT_CMPINT (1, ==, one);
+   for (i = 0; i < 2; i++) {
+      reader = bson_json_reader_new (multiple_json,
+                                     &test_bson_json_read_cb_helper,
+                                     NULL,
+                                     i == 1 /* allow_multiple */,
+                                     0);
 
-   /* read second json */
-   bson_reinit (&bson);
-   ASSERT_CMPINT (1, ==, bson_json_reader_read (reader, &bson, &error));
-   BCON_EXTRACT (&bson, "b", BCONE_INT32 (one));
-   ASSERT_CMPINT (1, ==, one);
+      BSON_ASSERT (reader);
 
-   /* start at the beginning of the string */
-   bson_reinit (&bson);
-   ASSERT_CMPINT (1, ==, bson_json_reader_read (reader, &bson, &error));
-   BCON_EXTRACT (&bson, "a", BCONE_INT32 (one));
-   ASSERT_CMPINT (1, ==, one);
+      /* read first json */
+      bson_init (&bson);
+      ASSERT_CMPINT (1, ==, bson_json_reader_read (reader, &bson, &error));
+      BCON_EXTRACT (&bson, "a", BCONE_INT32 (one));
+      ASSERT_CMPINT (1, ==, one);
 
-   /* reader_multiple with allow_multiple true */
-   /* read first json */
-   bson_reinit (&bson);
-   ASSERT_CMPINT (
-      1, ==, bson_json_reader_read (reader_multiple, &bson, &error));
-   BCON_EXTRACT (&bson, "a", BCONE_INT32 (one));
-   ASSERT_CMPINT (1, ==, one);
+      /* read second json */
+      bson_reinit (&bson);
+      ASSERT_CMPINT (1, ==, bson_json_reader_read (reader, &bson, &error));
+      BCON_EXTRACT (&bson, "b", BCONE_DBPOINTER (collection, oid));
+      ASSERT_CMPSTR (collection, "c");
+      BSON_ASSERT (0 == bson_oid_compare (oid, &oid_expected));
 
-   /* read second json */
-   bson_reinit (&bson);
-   ASSERT_CMPINT (
-      1, ==, bson_json_reader_read (reader_multiple, &bson, &error));
-   BCON_EXTRACT (&bson, "b", BCONE_INT32 (one));
-   ASSERT_CMPINT (1, ==, one);
+      /* read third json */
+      bson_reinit (&bson);
+      ASSERT_CMPINT (1, ==, bson_json_reader_read (reader, &bson, &error));
+      BCON_EXTRACT (&bson, "c", BCONE_INT32 (one));
+      ASSERT_CMPINT (1, ==, one);
 
-   /* start at the beginning of the string */
-   bson_reinit (&bson);
-   ASSERT_CMPINT (
-      1, ==, bson_json_reader_read (reader_multiple, &bson, &error));
-   BCON_EXTRACT (&bson, "a", BCONE_INT32 (one));
-   ASSERT_CMPINT (1, ==, one);
+      /* surprisingly, the reader begins at the start of the string again */
+      bson_reinit (&bson);
+      ASSERT_CMPINT (1, ==, bson_json_reader_read (reader, &bson, &error));
+      BCON_EXTRACT (&bson, "a", BCONE_INT32 (one));
+      ASSERT_CMPINT (1, ==, one);
 
-   bson_json_reader_destroy (reader);
-   bson_json_reader_destroy (reader_multiple);
-   bson_destroy (&bson);
+      bson_json_reader_destroy (reader);
+      bson_destroy (&bson);
+   }
 }
 
 
@@ -176,15 +157,28 @@ test_bson_as_json_multi (void)
    BSON_ASSERT (bson_append_double (b, "Double", -1, -1.0));
 
    {
-      const uint8_t binary[] = { 0xa3, 0x4c, 0x38, 0xf7, 0xc3, 0xab,
-                                 0xed, 0xc8, 0xa3, 0x78, 0x14, 0xa9,
-                                 0x92, 0xab, 0x8d, 0xb6 };
+      const uint8_t binary[] = {0xa3,
+                                0x4c,
+                                0x38,
+                                0xf7,
+                                0xc3,
+                                0xab,
+                                0xed,
+                                0xc8,
+                                0xa3,
+                                0x78,
+                                0x14,
+                                0xa9,
+                                0x92,
+                                0xab,
+                                0x8d,
+                                0xb6};
       BSON_ASSERT (bson_append_binary (
          b, "Binary", -1, BSON_SUBTYPE_UUID_DEPRECATED, binary, sizeof binary));
    }
 
    {
-      const uint8_t binary[] = { 1, 2, 3, 4, 5 };
+      const uint8_t binary[] = {1, 2, 3, 4, 5};
       BSON_ASSERT (bson_append_binary (
          b, "BinaryUserDefined", -1, BSON_SUBTYPE_USER, binary, sizeof binary));
    }
@@ -219,16 +213,18 @@ test_bson_as_json_multi (void)
    BSON_ASSERT (bson_append_timestamp (b, "Timestamp", -1, 42, 1));
    BSON_ASSERT (bson_append_regex (b, "Regex", -1, "pattern", ""));
    BSON_ASSERT (bson_append_date_time (b, "DatetimeEpoch", -1, 0));
-   BSON_ASSERT (bson_append_date_time (b, "DatetimePositive", -1, (int64_t) 2147483647LL));
-   BSON_ASSERT (bson_append_date_time (b, "DatetimeNegative", -1, (int64_t) -2147483648LL));
+   BSON_ASSERT (bson_append_date_time (
+      b, "DatetimePositive", -1, (int64_t) 2147483647LL));
+   BSON_ASSERT (bson_append_date_time (
+      b, "DatetimeNegative", -1, (int64_t) -2147483648LL));
    BSON_ASSERT (bson_append_bool (b, "True", -1, true));
    BSON_ASSERT (bson_append_bool (b, "False", -1, false));
 
    {
       bson_oid_t oid;
       bson_oid_init_from_string (&oid, "57e193d7a9cc81b4027498b1");
-      BSON_ASSERT (bson_append_dbpointer (
-         b, "DBPointer", -1, "collection", &oid));
+      BSON_ASSERT (
+         bson_append_dbpointer (b, "DBPointer", -1, "collection", &oid));
    }
 
    {
@@ -257,34 +253,40 @@ test_bson_as_json_multi (void)
    str = bson_as_json (b, &len);
 
    /* Based on multi-type-deprecated.json from BSON Corpus Tests. */
-   ASSERT_CMPSTR (str,
-                  "{"
-                  " \"_id\" : { \"$oid\" : \"57e193d7a9cc81b4027498b5\" },"
-                  " \"Symbol\" : \"symbol\","
-                  " \"String\" : \"string\","
-                  " \"Int32\" : 42,"
-                  " \"Int64\" : 42,"
-                  " \"Double\" : -1.0,"
-                  " \"Binary\" : { \"$binary\" : \"o0w498Or7cijeBSpkquNtg==\", \"$type\" : \"03\" },"
-                  " \"BinaryUserDefined\" : { \"$binary\" : \"AQIDBAU=\", \"$type\" : \"80\" },"
-                  " \"Code\" : { \"$code\" : \"function() {}\" },"
-                  " \"CodeWithScope\" : { \"$code\" : \"function() {}\", \"$scope\" : { } },"
-                  " \"Subdocument\" : { \"foo\" : \"bar\" },"
-                  " \"Array\" : [ 1, 2, 3, 4, 5 ],"
-                  " \"Timestamp\" : { \"$timestamp\" : { \"t\" : 42, \"i\" : 1 } },"
-                  " \"Regex\" : { \"$regex\" : \"pattern\", \"$options\" : \"\" },"
-                  " \"DatetimeEpoch\" : { \"$date\" : 0 },"
-                  " \"DatetimePositive\" : { \"$date\" : 2147483647 },"
-                  " \"DatetimeNegative\" : { \"$date\" : -2147483648 },"
-                  " \"True\" : true,"
-                  " \"False\" : false,"
-                  " \"DBPointer\" : { \"$ref\" : \"collection\", \"$id\" : \"57e193d7a9cc81b4027498b1\" },"
-                  " \"DBRef\" : { \"$ref\" : \"collection\", \"$id\" : { \"$oid\" : \"57fd71e96e32ab4225b723fb\" }, \"$db\" : \"database\" },"
-                  " \"Minkey\" : { \"$minKey\" : 1 },"
-                  " \"Maxkey\" : { \"$maxKey\" : 1 },"
-                  " \"Null\" : null,"
-                  " \"Undefined\" : { \"$undefined\" : true },"
-                  " \"Decimal128\" : { \"$numberDecimal\" : \"11\" } }");
+   ASSERT_CMPSTR (
+      str,
+      "{"
+      " \"_id\" : { \"$oid\" : \"57e193d7a9cc81b4027498b5\" },"
+      " \"Symbol\" : \"symbol\","
+      " \"String\" : \"string\","
+      " \"Int32\" : 42,"
+      " \"Int64\" : 42,"
+      " \"Double\" : -1.0,"
+      " \"Binary\" : { \"$binary\" : \"o0w498Or7cijeBSpkquNtg==\", \"$type\" : "
+      "\"03\" },"
+      " \"BinaryUserDefined\" : { \"$binary\" : \"AQIDBAU=\", \"$type\" : "
+      "\"80\" },"
+      " \"Code\" : { \"$code\" : \"function() {}\" },"
+      " \"CodeWithScope\" : { \"$code\" : \"function() {}\", \"$scope\" : { } "
+      "},"
+      " \"Subdocument\" : { \"foo\" : \"bar\" },"
+      " \"Array\" : [ 1, 2, 3, 4, 5 ],"
+      " \"Timestamp\" : { \"$timestamp\" : { \"t\" : 42, \"i\" : 1 } },"
+      " \"Regex\" : { \"$regex\" : \"pattern\", \"$options\" : \"\" },"
+      " \"DatetimeEpoch\" : { \"$date\" : 0 },"
+      " \"DatetimePositive\" : { \"$date\" : 2147483647 },"
+      " \"DatetimeNegative\" : { \"$date\" : -2147483648 },"
+      " \"True\" : true,"
+      " \"False\" : false,"
+      " \"DBPointer\" : { \"$ref\" : \"collection\", \"$id\" : "
+      "\"57e193d7a9cc81b4027498b1\" },"
+      " \"DBRef\" : { \"$ref\" : \"collection\", \"$id\" : { \"$oid\" : "
+      "\"57fd71e96e32ab4225b723fb\" }, \"$db\" : \"database\" },"
+      " \"Minkey\" : { \"$minKey\" : 1 },"
+      " \"Maxkey\" : { \"$maxKey\" : 1 },"
+      " \"Null\" : null,"
+      " \"Undefined\" : { \"$undefined\" : true },"
+      " \"Decimal128\" : { \"$numberDecimal\" : \"11\" } }");
 
    bson_free (str);
    bson_destroy (b);
@@ -638,7 +640,7 @@ test_bson_as_json_stack_overflow (void)
 
    buf = bson_malloc0 (buflen);
 
-   fd = bson_open (BINARY_DIR "/stackoverflow.bson", O_RDONLY);
+   fd = bson_open (BSON_BINARY_DIR "/stackoverflow.bson", O_RDONLY);
    BSON_ASSERT (-1 != fd);
 
    r = bson_read (fd, buf, buflen);
@@ -671,7 +673,7 @@ test_bson_corrupt (void)
 
    buf = bson_malloc0 (buflen);
 
-   fd = bson_open (BINARY_DIR "/test55.bson", O_RDONLY);
+   fd = bson_open (BSON_BINARY_DIR "/test55.bson", O_RDONLY);
    BSON_ASSERT (-1 != fd);
 
    r = bson_read (fd, buf, buflen);
@@ -699,7 +701,7 @@ test_bson_corrupt_utf8 (void)
 
    buf = bson_malloc0 (buflen);
 
-   fd = bson_open (BINARY_DIR "/test56.bson", O_RDONLY);
+   fd = bson_open (BSON_BINARY_DIR "/test56.bson", O_RDONLY);
    BSON_ASSERT (-1 != fd);
 
    r = bson_read (fd, buf, buflen);
@@ -727,7 +729,7 @@ test_bson_corrupt_binary (void)
 
    buf = bson_malloc0 (buflen);
 
-   fd = bson_open (BINARY_DIR "/test57.bson", O_RDONLY);
+   fd = bson_open (BSON_BINARY_DIR "/test57.bson", O_RDONLY);
    BSON_ASSERT (-1 != fd);
 
    r = bson_read (fd, buf, buflen);
@@ -743,6 +745,15 @@ test_bson_corrupt_binary (void)
    bson_free (buf);
 }
 
+#ifndef BSON_HAVE_RAND_R
+static int
+rand_r (unsigned int *seed)
+{
+   srand (*seed);
+   return rand ();
+}
+#endif
+
 #ifdef _WIN32
 #define RAND_R rand_s
 #else
@@ -835,7 +846,7 @@ test_bson_json_read_buffering (void)
          }
 
          /* finished parsing */
-         BSON_ASSERT_CMPINT (
+         ASSERT_CMPINT (
             0, ==, bson_json_reader_read (reader, &bson_out, &error));
 
          bson_json_reader_destroy (reader);
@@ -1034,6 +1045,33 @@ test_bson_json_read_corrupt_utf8 (void)
 }
 
 
+/* exercise early exit from _bson_as_json_visit_document/array, CDRIVER-2541 */
+static void
+test_bson_json_read_corrupt_document (void)
+{
+   /* like {a: {a: "\x80"}}, the value is invalid UTF-8 */
+   const char bad_doc[] = "\x16\x00\x00\x00" /* length */
+                          "\x03\x61\x00"     /* subdoc field with key "a" */
+                          "\x0e\x00\x00\x00" /* length of subdoc in bytes */
+                          "\x02\x61\x00\x02\x00\x00\x00\x80\x00" /* a: "\x80" */
+                          "\x00"; /* terminator */
+
+   /* like {a: ["\x80"]}, the inner value is invalid UTF-8 */
+   const char bad_array[] = "\x16\x00\x00\x00" /* length */
+                            "\x04\x61\x00"     /* array field with key "a" */
+                            "\x0e\x00\x00\x00" /* length of array in bytes */
+                            "\x02\x30\x00"     /* key "0" */
+                            "\x02\x00\x00\x00\x80\x00" /* string "\x80" */
+                            "\x00";                    /* terminator */
+
+   bson_t bson;
+   bson_init_static (&bson, (uint8_t *) bad_doc, sizeof (bad_doc));
+   BSON_ASSERT (!bson_as_json (&bson, NULL));
+   bson_init_static (&bson, (uint8_t *) bad_array, sizeof (bad_array));
+   BSON_ASSERT (!bson_as_json (&bson, NULL));
+}
+
+
 static void
 test_bson_json_read_decimal128 (void)
 {
@@ -1079,8 +1117,6 @@ test_bson_json_read_dbpointer (void)
       BSON_ASSERT (!r);
       ASSERT_ERROR_CONTAINS (
          error, BSON_ERROR_JSON, BSON_JSON_ERROR_READ_INVALID_PARAM, *(p + 1));
-
-      bson_destroy (&b);
    }
 }
 
@@ -1235,7 +1271,7 @@ test_bson_json_read_legacy_binary (void)
 static void
 test_json_reader_new_from_file (void)
 {
-   const char *path = JSON_DIR "/test.json";
+   const char *path = BSON_JSON_DIR "/test.json";
    const char *bar;
    const bson_oid_t *oid;
    bson_oid_t oid_expected;
@@ -1268,7 +1304,7 @@ test_json_reader_new_from_file (void)
 static void
 test_json_reader_new_from_bad_path (void)
 {
-   const char *bad_path = JSON_DIR "/does-not-exist";
+   const char *bad_path = BSON_JSON_DIR "/does-not-exist";
    bson_json_reader_t *reader;
    bson_error_t error;
 
@@ -1338,8 +1374,6 @@ test_bson_json_read_invalid_binary (void)
 
    r = bson_init_from_json (&b, json, -1, &error);
    BSON_ASSERT (!r);
-
-   bson_destroy (&b);
 }
 
 static void
@@ -1456,6 +1490,7 @@ test_bson_json_number_long (void)
    BSON_ASSERT (bson_iter_init_find (&iter, &b, "x"));
    BSON_ASSERT (BSON_ITER_HOLDS_INT64 (&iter));
    ASSERT_CMPINT64 (bson_iter_int64 (&iter), ==, (int64_t) INT64_MAX);
+   bson_destroy (&b);
 
    /* INT64_MIN */
    r = bson_init_from_json (
@@ -1914,8 +1949,6 @@ test_bson_json_uescape_bad (void)
                           BSON_ERROR_JSON,
                           BSON_JSON_ERROR_READ_CORRUPT_JS,
                           "UESCAPE_TOOSHORT");
-
-   bson_destroy (&b);
 }
 
 
@@ -2248,7 +2281,7 @@ test_bson_json_empty_final_object (void)
 {
    const char *json = "{\"a\": {\"b\": {}}}";
    bson_t *bson = BCON_NEW ("a", "{", "b", "{", "}", "}");
-   bson_t b = BSON_INITIALIZER;
+   bson_t b;
    bool r;
    bson_error_t error;
 
@@ -2394,6 +2427,7 @@ test_bson_json_array_subdoc (void)
    BSON_ASSERT (r);
 
    bson_eq_bson (&b, &compare);
+   bson_destroy (&subdoc);
    bson_destroy (&compare);
    bson_destroy (&b);
 }
@@ -2711,6 +2745,139 @@ test_bson_integer_width (void)
    bson_destroy (bs);
 }
 
+
+static void
+test_bson_json_null_in_str (void)
+{
+   const char bad_json[] = "{\"a\":\"\0\"}";
+   const char cdriver2305[] = "{\"\0";
+   bson_error_t err;
+   ASSERT (!bson_new_from_json (
+      (const uint8_t *) bad_json, sizeof (bad_json) - 1, &err));
+   ASSERT_ERROR_CONTAINS (
+      err, BSON_ERROR_JSON, BSON_JSON_ERROR_READ_CORRUPT_JS, "Got parse error");
+   ASSERT (!bson_new_from_json (
+      (const uint8_t *) cdriver2305, sizeof (cdriver2305) - 1, &err));
+   ASSERT_ERROR_CONTAINS (
+      err, BSON_ERROR_JSON, BSON_JSON_ERROR_READ_CORRUPT_JS, "Got parse error");
+}
+
+static char *
+_single_to_double (const char *json)
+{
+   char *copy;
+   char *to_ret = copy = bson_malloc (strlen (json) + 1);
+   strcpy (copy, json);
+   while (*copy) {
+      if (*copy == '\'') {
+         *copy = '\"';
+      } else {
+         copy++;
+      }
+   }
+   return to_ret;
+}
+
+static void
+_test_json_produces_multiple (const char *json_in, int err_expected, ...)
+{
+   bson_t bson_in = BSON_INITIALIZER;
+   bson_t *bson_expected;
+   bson_error_t err = {0};
+   int ret;
+   va_list ap;
+   char *json = _single_to_double (json_in);
+   bson_json_reader_t *reader =
+      bson_json_data_reader_new (false /* ignored */, 512 /* buffer size */);
+
+   va_start (ap, err_expected);
+
+   bson_json_data_reader_ingest (reader, (uint8_t *) json, strlen (json));
+   while ((ret = bson_json_reader_read (reader, &bson_in, &err)) == 1) {
+      bson_expected = va_arg (ap, bson_t *);
+      if (!bson_expected) {
+         fprintf (stderr, "Extra bson documents returned for input %s\n", json);
+         abort ();
+      }
+      if (bson_compare (&bson_in, bson_expected) != 0) {
+         char *expect = bson_as_json (bson_expected, NULL);
+         char *in = bson_as_json (&bson_in, NULL);
+         fprintf (
+            stderr, "Got %s, but expected %s for input %s\n", expect, in, json);
+         bson_free (expect);
+         bson_free (in);
+         abort ();
+      }
+      bson_destroy (bson_expected);
+      bson_reinit (&bson_in);
+   }
+
+   if (ret == 0) {
+      ASSERT (!err_expected);
+   } else {
+      ASSERT_OR_PRINT (err_expected, err);
+   }
+
+   if (va_arg (ap, bson_t *) != NULL) {
+      fprintf (stderr, "Not all bson docs matched for input %s\n", json);
+      abort ();
+   }
+
+   va_end (ap);
+
+   bson_json_reader_destroy (reader);
+   bson_destroy (&bson_in);
+   bson_free (json);
+}
+
+#define TEST_JSON_PRODUCES_MULTIPLE(_json, _has_err, ...) \
+   _test_json_produces_multiple (_json, _has_err, __VA_ARGS__, NULL);
+
+static void
+test_bson_as_json_multi_object (void)
+{
+   /* Test malformed documents that produce errors */
+   TEST_JSON_PRODUCES_MULTIPLE ("[],[{''", 1, NULL);
+
+   TEST_JSON_PRODUCES_MULTIPLE ("{},[{''", 1, NULL);
+
+
+   /* Test the desired multiple document behavior */
+   TEST_JSON_PRODUCES_MULTIPLE ("{'a': 1} {'b': 1}",
+                                0,
+                                BCON_NEW ("a", BCON_INT32 (1)),
+                                BCON_NEW ("b", BCON_INT32 (1)));
+
+   TEST_JSON_PRODUCES_MULTIPLE (
+      "{}{}{}", 0, bson_new (), bson_new (), bson_new ());
+
+   /* Test strange behavior we may consider changing */
+
+   /* Combines both documents into one? */
+   TEST_JSON_PRODUCES_MULTIPLE (
+      "{'a': 1}, {'b': 1}",
+      0,
+      BCON_NEW ("a", BCON_INT32 (1), "b", BCON_INT32 (1)));
+
+   TEST_JSON_PRODUCES_MULTIPLE (
+      "{},{'a': 1}", 0, BCON_NEW ("a", BCON_INT32 (1)));
+
+   TEST_JSON_PRODUCES_MULTIPLE (
+      "{},{},{'a': 1}", 0, BCON_NEW ("a", BCON_INT32 (1)));
+
+   TEST_JSON_PRODUCES_MULTIPLE (
+      "[],{'a': 1}", 0, BCON_NEW ("a", BCON_INT32 (1)));
+
+   /* We support a root level array */
+   TEST_JSON_PRODUCES_MULTIPLE ("[]", 0, bson_new ());
+
+   TEST_JSON_PRODUCES_MULTIPLE (
+      "[{'x': 0}]", 0, BCON_NEW ("0", "{", "x", BCON_INT32 (0), "}"));
+
+   /* Yet this fails */
+   TEST_JSON_PRODUCES_MULTIPLE ("[],[{'a': 1}]", 1, NULL);
+}
+
 void
 test_json_install (TestSuite *suite)
 {
@@ -2726,13 +2893,13 @@ test_json_install (TestSuite *suite)
                   test_bson_as_json_double_nonfinite);
 #endif
    TestSuite_Add (suite, "/bson/as_json/code", test_bson_as_json_code);
-   TestSuite_Add (suite, "/bson/as_json/date_time", test_bson_as_json_date_time);
+   TestSuite_Add (
+      suite, "/bson/as_json/date_time", test_bson_as_json_date_time);
    TestSuite_Add (suite, "/bson/as_json/regex", test_bson_as_json_regex);
    TestSuite_Add (suite, "/bson/as_json/symbol", test_bson_as_json_symbol);
    TestSuite_Add (suite, "/bson/as_json/utf8", test_bson_as_json_utf8);
-   TestSuite_Add (suite,
-                  "/bson/as_json/dbpointer",
-                  test_bson_as_json_dbpointer);
+   TestSuite_Add (
+      suite, "/bson/as_json/dbpointer", test_bson_as_json_dbpointer);
    TestSuite_Add (suite,
                   "/bson/as_canonical_extended_json/dbpointer",
                   test_bson_as_canonical_extended_json_dbpointer);
@@ -2777,6 +2944,9 @@ test_json_install (TestSuite *suite)
       suite, "/bson/json/read/raw_utf8", test_bson_json_read_raw_utf8);
    TestSuite_Add (
       suite, "/bson/json/read/corrupt_utf8", test_bson_json_read_corrupt_utf8);
+   TestSuite_Add (suite,
+                  "/bson/json/read/corrupt_document",
+                  test_bson_json_read_corrupt_document);
    TestSuite_Add (
       suite, "/bson/json/read/decimal128", test_bson_json_read_decimal128);
    TestSuite_Add (
@@ -2825,4 +2995,8 @@ test_json_install (TestSuite *suite)
       suite, "/bson/json/read/$numberDecimal", test_bson_json_number_decimal);
    TestSuite_Add (suite, "/bson/json/errors", test_bson_json_errors);
    TestSuite_Add (suite, "/bson/integer/width", test_bson_integer_width);
+   TestSuite_Add (
+      suite, "/bson/json/read/null_in_str", test_bson_json_null_in_str);
+   TestSuite_Add (
+      suite, "/bson/as_json/multi_object", test_bson_as_json_multi_object);
 }
